import os
import subprocess
import shutil

class ElectronicStructureCode():
    """Parent class defining the methods that each electronic structure code must offer"""
    def __init__(self,job_record,database):
        self._job_record = job_record
        self._database = database
        dir_name = 'dir_' + '{0:05d}'.format(self._job_record['id']) + '_' + self._job_record['descriptor']
        self._job_dir = os.path.join(self._job_record['folder'], dir_name)

    def make_directory(self):
       """Creates the directory in which the calculation is hosted"""
       try:
           os.makedirs(self._job_dir)
       except:
           print('Directory {} exists'.format(self._job_dir))
           raise RuntimeError

    def delete(self):
       try:
           shutil.rmtree(self._job_dir)
       except:
           print('Directory {} cannot be deleted.'.format(self._job_dir))
           raise RuntimeError

    def write_input(self):
        """Creates all input needed to execute the calculation in the desired directory"""
        try:
            self.make_directory()
        except: #this may happen if the directory exists for some reason
            print('Could not build directory for jobid {}'.format(self._job_record['id']))
            raise RuntimeError
            return
        #No input written for base class

    def run(self, recreate=False):
        """Executes the code"""
        homedir = os.getcwd()
        try:
            os.chdir(self._job_dir)
        except:
            print('Direcory {} does not exist. Something strange must have happened'.format(self._job_dir))
            raise RuntimeError
        execute_statement = self.execute()
        job_script ="""#!/bin/bash
#SBATCH --nodes={nnodes} #1
#SBATCH --ntasks={ncpu} #128
#SBATCH --partition={partition}
#SBATCH --time={runhours}:00:00 #01:00:00

# Create folder and copy input to scratch.
WORKDIR=${{SLURM_SUBMIT_DIR}}
SCRATCH=/scratch-shared/${{SLURM_JOB_USER}}/${{SLURM_JOB_ID}}
echo {{$SCRATCH}}
echo {{$WORKDIR}}
mkdir -p $SCRATCH
echo  ""
echo "copying"
rsync -Lhvrt ${{WORKDIR}}/  ${{SCRATCH}}/
cd ${{SCRATCH}}

# Execute the program
{execute_statement}


# Copy output back from scratch to the directory from where the job was submitted
echo "copying back"
rsync -hvrt --update  ${{SCRATCH}}/ ${{WORKDIR}}/
rm -r ${{SCRATCH}}/*
rm -r ${{SCRATCH}}

cd ${{WORKDIR}}
rm .runlock_autogen
touch .completed_autogen

""".format(nnodes=self._job_record['nnodes'], ncpu=self._job_record['ncpu'], partition=self._job_record['partition'], runhours=self._job_record['run hours'], execute_statement=execute_statement)
        job_script_name = 'job_script'
        f=open(job_script_name,'w')
        f.write(job_script)
        f.close

        if not recreate==True: #only run if not meant to only recreate relevant input
            f=open(".runlock_autogen", 'w')
            f.write("running")
            f.close()
    
            proc = subprocess.Popen(['sbatch', job_script_name], stdout=subprocess.PIPE)
            output=proc.communicate()[0]#.split()
            #output=subprocess.check_output(["qsub", jobToSubmit])
            self._job_record['job number']=[int(s) for s in output.split() if s.isdigit()][0]
        os.chdir(homedir)


    def execute(self):
        """Returns a string with the command to execute the code"""
        return "\n" #blank for parent class

    def get_status(self):
        """Gets the status of the calculation"""
        
        #check if job is done, in this case, get_status of each code needs to take over
        filename = os.path.join(self._job_dir, '.completed_autogen')
        if os.path.exists(filename):
            return 'executed'

        #check if running
        filename = os.path.join(self._job_dir, '.runlock_autogen')
        if os.path.exists(filename):
            proc = subprocess.Popen(['squeue','-j',str(self._job_record['job number'])], stdout=subprocess.PIPE)
            output=proc.communicate()[0]
            if " R " in str(output):
                self._job_record['status'] = 'running'
                return 'running'
            elif " PD " in str(output):
                self._job_record['status'] = 'pending'
                return 'pending' 
            else:
                self._job_record['status'] = 'error'
                return 'error'

        #check if prepared
        elif os.path.exists(self._job_dir):
            self._job_record['status'] = 'prepared'
            return 'prepared'

        #else: new
        else: 
            self._job_record['status'] = 'new'
            return 'new'


    def clean(self):
        """Cleans out all files that are (typically) not needed and may be large"""
        return #doesn't do anything for parent class

    def get_job_record(self):
        return self._job_record
