import os
from json import dumps
from electronicStructureCode import ElectronicStructureCode
import shutil
import ase
import ase.io
from numbers import Number

class Vasp(ElectronicStructureCode):
    """Class to initiate and run Vasp6 calculations"""
    def __init__(self,job_record,database):
        super().__init__(job_record,database)

    def write_input(self, recreate = False):
        """Creates all input needed to execute the calculation in the desired directory""" 
        try:
            super().write_input() #should throw an exception if path exists for some reason
        except:
            return #if path exists, don't do anything (may othewise overwrite stuff)
        try:
            self.write_poscar()
            if "4step" in self._job_record['xc']: 
                self.write_incar(algo='dft', filename='INCAR_DFT')
                self.write_incar(algo='exact', filename='INCAR_EXACT')
                self.write_incar(algo='rpar', filename='INCAR_ACFDT')
            else:
                if self._job_record['xc'] == 'rpar':
                    self.write_incar(algo='rpar', filename='INCAR')
                elif 'hse' in self._job_record['xc']:
                    self.write_incar(algo='dft', filename='INCAR_DFT')
                    self.write_incar(algo='hybrid', filename='INCAR_HYBRID')
                elif 'm06l' in self._job_record['xc']: #M06L needs special treatment to converge
                    self.write_incar(algo='dft', filename='INCAR_PBE')
                    self.write_incar(algo='difficultMeta', filename='INCAR_M06L')
                else: #standard dft calculation
                    self.write_incar(algo='dft', filename='INCAR')
            self.write_kpoints()
            self.write_potcar(recreate)
            if "vdw" in self._job_record['xc']:
                self.write_vdw_kernel()
        except Exception as error:
            print("Something went wrong in writing the input for {}".format(self._job_dir))
            print(error)
            try:
                shutil.rmtree(self._job_dir) #delete directory if input creation not successful
            except OSError as e:
                print("Error: %s : %s" % (dir_path, e.strerror))

    def execute(self):
        """Returns a string with the command to execute the code"""
        if self._job_record['program']=="vasp6.2":
            load_text = "module load 2021\nmodule load VASP6/6.2.1-foss-2021a-CUDA-11.3.1"
        elif self._job_record['program']=="vasp6.3.0":
            load_text = "module load 2021\nmodule load VASP6/6.3.0-foss-2021a-CUDA-11.3.1"
        elif self._job_record['program']=="vasp6.3.2":
            load_text = "module load 2022\nmodule load VASP6/6.3.2-foss-2022a" 
        elif self._job_record['program']=="vasp6.3.0-beef":
            load_text = "module load 2021\nmodule load VASP6/6.3.0-foss-2021a-beef"
        execute_statement="""# Load modules for MPI and other parallel libraries
{load_text}
#unset non-buffer write for better performance (suggested by Surf support)
export GFORTRAN_UNBUFFERED_ALL=no
""".format(load_text=load_text)
        if "4step" in self._job_record['xc']:
            execute_statement+="""#run
check_output () {{
        if [ ! -f OUTCAR ]; then
                return 55
        fi
        re='^[0-9]+$'
        xxx=$(grep "maximum number of plane-waves" OUTCAR | tail -1 |  awk '{{{{print $5}}}}')
        xxx=$(echo "$xxx/{ncpu}*{ncpu}"|bc)
        if ! [[ $xxx =~ $re ]]; then
                return 55
        fi
        if grep -q "General timing" OUTCAR; then
                return 0
        else
                return 55
        fi

}}

# Execute the program in parallel on ntasks cores
cp INCAR_DFT INCAR
srun vasp_std
check_output
status=$?
echo "DFT status:", $status
if test $status -eq 0
then
        xxx=$(grep "maximum number of plane-waves" OUTCAR | tail -1 |  awk '{{{{print $5}}}}')
        xxx=$(echo "$xxx/{ncpu}*{ncpu}"|bc)
        mv OSZICAR OSZICAR_DFT
        mv OUTCAR OUTCAR_DFT

        sed -i "s/xxx/$xxx/" INCAR_EXACT
        cp INCAR_EXACT INCAR
        srun vasp_std
fi
check_output
status=$?
echo "Exact diag. status:", $status
if test $status -eq 0
then
        mv OUTCAR OUTCAR_EXACT

        sed -i "s/xxx/$xxx/" INCAR_ACFDT
        cp INCAR_ACFDT INCAR
        srun vasp_std

        rm CHGCAR WAVECAR
fi
""".format(ncpu=self._job_record['ncpu'])

        elif "hse" in self._job_record['xc']: #hybrid calc
            execute_statement+="""#run
check_output () {{
        if [ ! -f OUTCAR ]; then
                return 55
        fi
        re='^[0-9]+$'
        xxx=$(grep "maximum number of plane-waves" OUTCAR | tail -1 |  awk '{{{{print $5}}}}')
        xxx=$(echo "$xxx/{ncpu}*{ncpu}"|bc)
        if ! [[ $xxx =~ $re ]]; then
                return 55
        fi
        if grep -q "General timing" OUTCAR; then
                return 0
        else
                return 55
        fi

}}

# Execute the program in parallel on ntasks cores
cp INCAR_DFT INCAR
srun vasp_std
check_output
status=$?
echo "DFT status:", $status
if test $status -eq 0
then
        mv OSZICAR OSZICAR_DFT
        mv OUTCAR OUTCAR_DFT

        cp INCAR_HYBRID INCAR
        srun vasp_std

        rm CHGCAR WAVECAR
fi
""".format(ncpu=self._job_record['ncpu'])

        elif self._job_record['xc']=='m06l': #some meta-GGA calcs that need special treatment to converge; PBE first, then meta-GGA (e.g., M06L). These need to be listed here!
            execute_statement+="""#run
check_output () {{
        if [ ! -f OUTCAR ]; then
                return 55
        fi
        re='^[0-9]+$'
        xxx=$(grep "maximum number of plane-waves" OUTCAR | tail -1 |  awk '{{{{print $5}}}}')
        xxx=$(echo "$xxx/{ncpu}*{ncpu}"|bc)
        if ! [[ $xxx =~ $re ]]; then
                return 55
        fi
        if grep -q "General timing" OUTCAR; then
                return 0
        else
                return 55
        fi

}}

# Execute the program in parallel on ntasks cores
cp INCAR_PBE INCAR
srun vasp_std
check_output
status=$?
echo "DFT status:", $status
if test $status -eq 0
then
        mv OSZICAR OSZICAR_PBE
        mv OUTCAR OUTCAR_PBE

        cp INCAR_M06L INCAR
        srun vasp_std

        rm CHGCAR WAVECAR
fi
""".format(ncpu=self._job_record['ncpu'])


        else: #"normal" submit (only one call to vasp necessary)
            execute_statement+="""#run
srun vasp_std
"""
        return execute_statement

    def get_status(self):
        """Returns the energy of a successful calculation and a detailed status"""
        status = super().get_status()

        if status == 'executed': #executed --> need to extract energy and set on done, error, or not_converged
            scf_converged = False
            geom_converged = False
            dft_energy = 0.0
            rpa_energy = 0.0
            rpa_xc_corr_energy = 10000
            #search for convergence criteria and for energies
            if '4step' in self._job_record['xc']:
                filenames = [os.path.join(self._job_dir, 'OUTCAR_DFT'), os.path.join(self._job_dir, 'OUTCAR_EXACT'),os.path.join(self._job_dir, 'OUTCAR')]
            else:
                filenames = [os.path.join(self._job_dir, 'OUTCAR')]
            for filename in filenames:
                try:
                    with open(filename) as f:
                        for line in f:
                            if "aborting loop because EDIFF is reached" in line:
                                scf_converged = True
                            if "energy without entropy" in line:
                                scf_converged = False #needs to find an instance of "EDIFF is reached" AFTER this energy in order for it to be converged
                                dft_energy = float(line.split()[7])
                            if "HF+E_corr(extrapolated)" in line:
                                rpa_energy = float(line.split()[2])
                            if " ACFDT corr.    EXHF_ACFDT" in line:
                                rpa_xc_corr_energy = float(line.split()[4])
                            if "reached required accuracy - stopping structural" in line:
                                geom_converged = True
                except:
                    pass
            if self._job_record['scf'] == 'single point': #for single point calculations: no geom opt
                geom_converged = True
            #select dft or rpa energy as requested    
            if 'rpa' in self._job_record['xc']:
                if '6.1' in self._job_record['program']: #VASP 6.1 adds the correction term directly in.
                    if rpa_xc_corr_energy == 10000: #not found
                        energy = 0.0
                    else:
                        energy = rpa_energy - rpa_xc_corr_energy 
                else: 
                    energy = rpa_energy
            else:
                energy = dft_energy
            #set energy and status 
            if (scf_converged and geom_converged and not energy == 0.0):
                self._job_record['energy'] = energy
                if 'rpa' in self._job_record[ 'xc']:
                    self._job_record['rpa_xc_corr'] = rpa_xc_corr_energy
                status = 'done'
                self.clean()
                self.collect_warnings()
            elif (scf_converged and not energy == 0.0):
                status = 'opt_not_converged'
            elif (not energy == 0.0):
                status = 'scf_not_converged'
            else:
                status = 'error'
            self._job_record['status'] = status

        return status



    def clean(self):
        """Cleans out all files that are (typically) not needed and may be large"""
        for i in ["CHG","CHGCAR","CONTCAR","DOSCAR","EIGENVAL","IBZKPT","PCDAT","REPORT","vasprun.xml","WAVECAR","XDATCAR"]:
            filepath=os.path.join(self._job_dir, i)
            try:
                os.remove(filepath)
            except:
                pass

    def write_poscar(self):
        filename = os.path.join(self._job_dir, "POSCAR")
        f = open(filename, 'w')
        f.write(self._job_record['geom'])
        f.close()

    def write_incar(self, algo, filename):
        """writes an incar file for the use in vasp depending on the input in the database"""
        #first get some input dependent parameters:
        #single point or optimization?
        if self._job_record['scf'] == 'single point':
            opt_info = '' 
        elif self._job_record['scf'] == 'opt':
            opt_info = """ NELMIN = 6\n NSW = 100 \n IBRION = 1\n ISIF = 2\n EDIFFG = -1E-3\n"""
        else:
            print("Chosen scf calculation ({}) not implemented (jobid: {})".format(self._job_record['scf'],self._job_record['id']))
            raise ValueError

        #DFT or RPA?
        if algo == 'rpar':
            algo_name = "RPAR"
            ediff = 1.E-8
            lwave = ".FALSE."
            lcharg =  ".FALSE."
        elif algo == 'hybrid':
            algo_name = "d"
            ediff = 1.E-4
            lwave = ".FALSE."
            lcharg =  ".FALSE."
        elif algo == 'dft' and '4step' in self._job_record['xc']: #DFT step for 4step rpa
            algo_name = "Normal"
            ediff = 1.E-8
            lwave = ".TRUE."
            lcharg =  ".TRUE."
        elif algo == 'dft' and 'hse' in self._job_record['xc']: #DFT calc preparing for a hybrid calc
            algo_name = "Normal"
            ediff = 1.E-6
            lwave = ".TRUE."
            lcharg =  ".TRUE."
        elif algo == 'dft' and 'm06l' in self._job_record['xc']: #DFT calc preparing for a m06l  calc
            algo_name = "F"
            ediff = 1.E-6
            lwave = ".TRUE."
            lcharg =  ".TRUE."
        elif algo == 'difficultMeta' and isinstance(self._job_record['damp'],Number): #meta-GGA step for calc for difficult meta's such as M06L using damping alg
            algo_name = "d"
            ediff = 1.E-4
            lwave = ".FALSE."
            lcharg =  ".FALSE."
        elif algo == 'difficultMeta': #meta-GGA step for calc for difficult meta's such as M06L
            algo_name = "all"
            ediff = 1.E-4
            lwave = ".FALSE."
            lcharg =  ".FALSE."
        elif algo == 'exact':
            algo_name = "Exact"
            ediff = 1.E-8
            lwave = ".TRUE."
            lcharg =  ".TRUE."
        else: #"normal" DFT calculation
            algo_name = "F"
            ediff = 1.E-6
            lwave = ".FALSE."
            lcharg =  ".FALSE."

        #additional info if RPA
        rpa_info = ""
        if algo == 'rpar':
            rpa_info = "PRECFOCK = Normal\n LOPTICS = .FALSE.\n"
            if self._job_record['finite temperature']:
                    rpa_info += """ LFINITE_TEMPERATURE = .TRUE.\n"""
            if not self._job_record['nomega'] == None:
                rpa_info += """ NOMEGA = {}\n""".format(self._job_record['nomega'])
            if '4step' in self._job_record['xc']:
                rpa_info += " NBANDS = xxx\n"

        if self._job_record['vasp_ncore'] == None:
             ncore = int(self._job_record['ncpu']/2)
        else:
            ncore = self._job_record['vasp_ncore']

        #additional info if hybrid calculation
        hybrid_info = ""
        if algo == 'hybrid':
            hybrid_info = "TIME = 0.2\n LSUBROT = .TRUE.\n LHFCALC = .TRUE.\n"
            if self._job_record['xc'] == 'hse06':
                hybrid_info += " GGA = PE\n HFSCREEN = 0.2\n NCORE = {}\n".format(ncore) #NCORE setting recommented by surf support
            elif 'hse06x' in self._job_record['xc']:
                aexx = float(self._job_record['xc'].split('x')[1])
                hybrid_info += " GGA = PE\n HFSCREEN = 0.2\n AEXX = {}\n NCORE = {}\n".format(aexx, ncore) #NCORE setting recommented by surf support
            else:
                print("Chosen xc functional ({}) not implemented (jobid: {})".format(self._job_record['xc'], self._job_record['id']))
                raise ValueError

        #additional info difficult meta-GGA runs that need PBE preparation
        difficult_meta_info = ""
        if algo == 'difficultMeta':
            difficult_meta_info += " METAGGA = {}\n LMIXTAU = .TRUE.\n NELMDL = 0\n NCORE = {}\n".format(self._job_record['xc'].upper(),ncore)

        #additional info exact
        exact_info = ""
        if algo == 'exact':
            exact_info = "LOPTICS = .FALSE.\n NELM =1\n NBANDS = xxx\n NCORE = {}\n".format(ncore) #NCORE setting recommented by surf support


        #additional info DFT
        dft_info = ""
        if  algo == 'dft':
            #RPA (4 step)
            if '4step' in self._job_record['xc']: #DFT step for 4step rpa
                dft_info = "NCORE = {}\n".format(ncore) #NCORE setting recommented by surf support
            #HYBRID (2 step)
            elif 'hse' in self._job_record['xc']: #DFT step for a hybrid calculation (2step)
                dft_info = """GGA = PE \n NCORE = {}\n""".format(ncore) #PE is the base functional for HSE06; NCORE setting recommented by surf support
            #PBE
            elif self._job_record['xc'] == 'pbe': #PBE
                dft_info = """GGA = PE \n NCORE = {}\n""".format(ncore) #NCORE setting recommented by surf support
            #RPBR
            elif self._job_record['xc'] == 'rpbe': #PBE
                dft_info = """GGA = RP \n NCORE = {}\n""".format(ncore) #NCORE setting recommented by surf support
            #PBE-vdW-DF
            elif self._job_record['xc'] == 'pbe-vdw-df': #PBE-vdW-DF
                dft_info = """GGA = PE \n LUSE_VDW = .TRUE. \n AGGAC = 0.0000 \n NCORE = {}\n""".format(ncore) #NCORE setting recommented by surf support
            #PBE-vdW-DF2
            elif self._job_record['xc'] == 'pbe-vdw-df2': #PBE-vdW-DF
                dft_info = """GGA = PE \n LUSE_VDW = .TRUE. \n ZAB_VDW = -1.886700\n AGGAC = 0.0000 \n NCORE = {}\n""".format(ncore) #NCORE setting recommented by surf support
            #RPBE-vdW-DF
            elif self._job_record['xc'] == 'rpbe-vdw-df': #RPBE-vdW-DF
                dft_info = """GGA = RP \n LUSE_VDW = .TRUE. \n AGGAC = 0.0000 \n NCORE = {}\n""".format(ncore) #NCORE setting recommented by surf support
            #BEEF-vdW
            elif self._job_record['xc'] == 'beef-vdw': #BEEF-vdW (uses DF2, but also mixes in GGA correlation)
                dft_info = """GGA = BF \n LUSE_VDW = .TRUE. \n ZAB_VDW = -1.886700 \n NCORE = {}\n""".format(ncore) #NCORE setting recommented by surf support
            #SCAN
            elif self._job_record['xc'] == 'scan': #SCAN
                dft_info = """METAGGA = SCAN \n NCORE = {}\n""".format(ncore) #NCORE setting recommented by surf support
            #revTPSS
            elif self._job_record['xc'] == 'revtpss': #revTPSS
                dft_info = """METAGGA = RTPSS \n NCORE = {}\n""".format(ncore) #NCORE setting recommented by surf support
            #M06L: DFT preparation run
            elif self._job_record['xc'] == 'm06l': #M06L, but this is the PBE preparation run
                dft_info = """GGA = PE \n NCORE = {}\n""".format(ncore) #NCORE setting recommented by surf support
                #dft_info = """METAGGA = M06L \n NCORE = {}\n""".format(ncore) #NCORE setting recommented by surf support
            #not supported
            else:
                print("Chosen xc functional ({}) not implemented (jobid: {})".format(self._job_record['xc'], self._job_record['id']))
                raise ValueError

        #additional info dipole correction
        dipole_info = ""
        if 'dipolecorr' in self._job_record:
            if not (self._job_record['dipolecorr']==None or self._job_record['dipolecorr']<1):
                f=open('tmp', 'w')
                f.write(self._job_record['geom'])
                f.close()
                geomInt=ase.io.read('tmp', format='vasp')
                os.remove('tmp')
                [xcom, ycom, zcom]=geomInt.get_center_of_mass()
                cell=geomInt.get_cell()
                zlatticev=cell[2,2]
                dipole_info+=""" IDIPOL     = {idipol}
 LDIPOL      =.TRUE.
 DIPOL       = 0.0 0.0 {zcom}
 LVTOT       =.True.\n""".format(idipol=self._job_record['dipolecorr'], zcom=zcom/zlatticev)

        #additional info parallelization
        parallelization_info = ""
        if 'kpar' in self._job_record:
            if not (self._job_record['kpar']==None or self._job_record['kpar']<1):
                parallelization_info = " KPAR = {}\n".format(self._job_record['kpar'])

        #smearing info: 
        #get smearing tag
        if self._job_record['smearing'] == 'fermi':
            ismear = -1
        elif self._job_record['smearing'] == 'gaussian':
            ismear = 0
        elif self._job_record['smearing'] == 'mp':
            ismear = 2
        elif self._job_record['smearing'] == 'tetrahedron':
            ismear = -5
        else:
            print("Chosen smearing type ({}) not implemented (jobid: {})".format(\
                    self._job_record['smearing'], \
                    self._job_record['id']))
            raise ValueError
        #write smearing info
        if algo == 'rpar':
            if self._job_record['finite temperature']:
                if not  ismear == -1:
                    print("For finite temperature RPA, smearing type must be Fermi (jobid: {})".format(\
                            self._job_record['id']))
                    raise ValueError
                smearing = """ISMEAR = -1\n SIGMA = {sigma}\n""".format(sigma=self._job_record['smearing val'])
            else:
                smearing = """ISMEAR = {smearing}\n SIGMA = {sigma}\n""".format(smearing = ismear, \
                        sigma=self._job_record['smearing val'])
        elif self._job_record['smearing'] == 'tetrahedron': #no smearing value needed
            smearing = """ISMEAR = {smearing}\n """.format(smearing = ismear)
        else :
            smearing = """ISMEAR = {smearing}\n SIGMA = {sigma}\n""".format(smearing = ismear, \
                    sigma=self._job_record['smearing val'])
        #memory info:
        if self._job_record['partition'] == 'thin':
            maxmem = int(240000./128.)
        elif self._job_record['partition'] == 'rome':
            maxmem = int(240000./128.)
        elif self._job_record['partition'] == 'fat':
            maxmem = int(1000000./128.)
        elif  self._job_record['partition'] == 'himem_4tb':
            maxmem = int(4000000./128.)
        elif  self._job_record['partition'] == 'himem_8tb':
            maxmem = int(8000000./128.)
        else: 
            print("Chosen partition ({}) not implemented (jobid: {})".format(\
                    self._job_record['partition'], \
                    self._job_record['id']))
            raise ValueError
        memory_info = """MAXMEM = {} \n """.format(maxmem-200)

        #additional info iterations, etc:
        misc_info = ""
        if not algo == 'exact': #don't add number of scf-iteractions if exact diagonalization is asked (exact diag is non-self-consistent!)
            if 'scf_iterations' in self._job_record:
                if not (self._job_record['scf_iterations']==None or self._job_record['scf_iterations']<1):
                    misc_info+=""" NELM     = {}\n""".format(self._job_record['scf_iterations'])
        if 'ngx' in self._job_record:
            if not self._job_record['ngx']==None:
                misc_info+=""" NGX    = {}\n""".format(self._job_record['ngx'])
        if 'ngy' in self._job_record:
            if not self._job_record['ngy']==None:
                misc_info+=""" NGY    = {}\n""".format(self._job_record['ngy'])
        if 'ngz' in self._job_record:
            if not self._job_record['ngz']==None:
                misc_info+=""" NGZ    = {}\n""".format(self._job_record['ngz'])
        if isinstance(self._job_record['damp'],Number):
            misc_info+=""" time    = {}\n""".format(self._job_record['damp'])

        #now write the incar
        incar = """ {name}

 LWAVE = {lwave}
 LCHARG = {lcharg}

 ALGO = {algo_name}
 {additional_rpa}{additional_dft}{additional_meta}{additional_opt}{additional_exact}{additional_hybrid}{additional_parallelization}{additional_dipole}{additional_misc}
 ENCUT = {encut}
 EDIFF = {ediff}

 {smearing}
 LASPH = .TRUE.

 {memory}
        """.format(name = self._job_record['folder']+'/'+self._job_record['descriptor'],\
                lwave = lwave,\
                lcharg = lcharg,\
                algo_name=algo_name,\
                encut=self._job_record['ecut'],\
                ediff = ediff,\
                additional_rpa = rpa_info,\
                additional_dft = dft_info,\
                additional_meta = difficult_meta_info,\
                additional_opt = opt_info,\
                additional_exact = exact_info,\
                additional_hybrid = hybrid_info,\
                additional_parallelization = parallelization_info,\
                additional_dipole = dipole_info,\
                additional_misc = misc_info,\
                smearing = smearing,\
                memory = memory_info)

        filename = os.path.join(self._job_dir, filename)
        f = open(filename, 'w')
        f.write(incar)
        f.close()

    def write_potcar(self, recreate):
        if recreate: #do NOT search for the file defined in 'pseudofiles'. Instead print the info available in 'pseudopotentials'
            filename = os.path.join(self._job_dir, 'POTCAR_info')
            f = open(filename, 'w')
            f.write(self._job_record['pseudopotentials'])
            f.close()
        else:
            potcar = os.path.join(self._job_dir, "POTCAR")
            homedir = os.getcwd()
            potcar_from = os.path.join(homedir, self._job_record['pseudofiles'])
            os.symlink(potcar_from, potcar)
            #the following puts the name of the PPs present in the file into the database
            pseudos=[]
            filename = os.path.join(self._job_dir, 'POTCAR')
            with open(filename) as f:
                for line in f:
                    if "TITEL" in line:
                        pseudos.append(line.split("= ")[1])
            pseudos_json_string = dumps(pseudos)
            self._job_record['pseudopotentials'] = pseudos_json_string
    def write_vdw_kernel(self):
        kernel = os.path.join(self._job_dir, "vdw_kernel.bindat")
        if self._job_record['program']=="vasp6.2":
            bindir = "<path-to-vasp6.2-bindir>/bin/"
        elif self._job_record['program']=="vasp6.3.0":
            bindir = "<path-to-vasp6.3.0-bindir>/bin/"
        elif self._job_record['program']=="vasp6.3.0":
            bindir = "<path-to-vasp6.3.0-bindir>/bin/"
        elif self._job_record['program']=="vasp6.3.0-beef":
            bindir = "<path-to-vasp6.3.0-with-beef-bindir>/bin/"
        kernel_from = os.path.join(bindir, "vdw_kernel.bindat")
        os.symlink(kernel_from, kernel)

    def write_kpoints(self):
        kpoints = """kpoints
0
Gammacentered
{kx} {ky} {kz}
0.0 0.0 0.0
""".format(kx = self._job_record['nkx'],\
                ky = self._job_record['nky'],\
                kz = self._job_record['nkz'])

        filename = os.path.join(self._job_dir, "KPOINTS")
        f = open(filename, 'w')
        f.write(kpoints)
        f.close()

    def collect_warnings(self):
        filename = os.path.join(self._job_dir, "OUTCAR")
        i=0
        warning = False
        warnings = []
        if '4step' in self._job_record['xc']:
            filenames = [os.path.join(self._job_dir, 'OUTCAR_DFT'), os.path.join(self._job_dir, 'OUTCAR_EXACT'),os.path.join(self._job_dir, 'OUTCAR')]
        elif 'hse' in self._job_record['xc']:
            filenames = [os.path.join(self._job_dir, 'OUTCAR_DFT'), os.path.join(self._job_dir, 'OUTCAR')]
        else:
            filenames = [os.path.join(self._job_dir, 'OUTCAR')]
        for filename in filenames:
            try:
                with open(filename) as f:
                    for line in f:
                        #if a warning is located in the output, collect the first sentence of th warning message
                        if warning:
                            i += 1
                            if i > 3: #need to skip first 3 lines to get to the start of the warning message
                                if "optimal performance we recommend" in line: #do not put into warnings
                                    warning = False
                                else:
                                    full_stops = [i for i, s in enumerate(line.split()) if '.' in s]
                                    if len(full_stops) > 0:
                                        end = full_stops[0]+1
                                    else:
                                        end = -1
                                    warning_text += " ".join(line.split()[1:end])
                                    if end > 0:
                                        warning = False
                                        warnings.append(warning_text)
                            if i > 5: #never collect more than 2 lines of warning text.
                                warning = False
                                warnings.append(warning_text)
                        if "AAAAAA" in line: #indicator that a warning is given
                            i=0
                            warning = True
                            warning_text = ""
            except:
                pass

        warnings_json_string = dumps(warnings)
        self._job_record['warnings']=warnings_json_string




        
        



    
